from enum import Enum


class ExpensesHeaders(Enum):
    PRODUCTO = "producto"
    EGRESOS = "egresos"
    FECHA = "fecha"


class TasksHeaders(Enum):
    DONE = "Done"
    TASK = "Task"
    FOCUS_TIME = "Focus time"
    DUE_DATE = "Due date"
    TAGS = "Tags"
    TICKTICK_ID = "Ticktick id"
    RECURRENT_ID = "Recurrent id"
    PROJECT_ID = "Project id"
    TIMEZONE = "Timezone"


class StatsHeaders(Enum):
    COMPLETED = "completed"
    DATE = "date"
    WORK_TIME = "work time"
    FOCUS_TIME = "focus time"
    LEISURE_TIME = "leisure time"
    WEIGHT = "weight"



from attrs import define


@define
class TimeStats:
    """ Represents the time stats of a personal stats row in Notion.

    Attributes:
        work_time: Personal work time.
        leisure_time: Personal leisure time.
        focus_time: Personal focus time.
    """
    work_time: float
    leisure_time: float
    focus_time: float


@define
class PersonalStats:
    """Represents a personal stats row in Notion.

    Attributes:
        date: The date of the stats in format YYYY-MM-DD.
        time_stats: The time stats of the row.
        weight: Personal weight.
    """
    date: str
    time_stats: TimeStats
    weight: float = 0.0



import json
import random
from typing import List, Dict

import pytest
from nothion._config import NT_TASKS_DB_ID, NT_STATS_DB_ID, NT_EXPENSES_DB_ID

from nothion._notion_api import NotionAPI
from nothion._notion_payloads import NotionPayloads
from .conftest import TEST_TASK, TEST_STAT, TEST_EXPENSE_LOG, EXISTING_TEST_TASK_PAGE_ID, EXISTING_TEST_STAT_PAGE_ID, \
    EXISTING_TEST_EXPENSE_LOG_PAGE_ID


@pytest.fixture(scope="module")
def notion_api(notion_info):
    return NotionAPI(notion_info["auth_secret"])


@pytest.mark.parametrize("payload", [
    # Test with tasks database
    (NotionPayloads.create_task(TEST_TASK)),

    # Test with stats database
    (NotionPayloads.create_stat_row(TEST_STAT)),

    # Test with expenses database
    (NotionPayloads.create_expense_log(TEST_EXPENSE_LOG)),
])
def test_create_table_entry(notion_api, payload):
    table_entry_data = notion_api.create_table_entry(payload)

    table_entry = notion_api.get_table_entry(table_entry_data["id"])

    notion_api.update_table_entry(table_entry["id"], NotionPayloads.delete_table_entry())


@pytest.mark.parametrize("database_id, query, expected_property, expected_value", [
    # Test with tasks database
    (NT_TASKS_DB_ID,
     {"filter": {"and": [{"property": "Ticktick id", "rich_text": {"equals": "a7f9b3d2c8e60f1472065ac4"}}]}},
     "id", "f0889936-35c3-40cc-8e98-298ab93ed685"),

    # Test with stats database
    (NT_STATS_DB_ID,
     {"filter": {"and": [{"property": "name", "rich_text": {"equals": "Test Existing Stat"}}]}},
     "id", "c568738e-82a2-4b25-8071-e5412db89a2f"),

    # Test with expenses database
    (NT_EXPENSES_DB_ID,
     {"filter": {"and": [{"property": "producto", "rich_text": {"equals": "Test Existing Expense Log"}}]}},
     "id", "36de61f8-b24c-49e2-86bb-5b0aca9740ab"),
])
def test_query_table(notion_api, database_id, query, expected_property, expected_value):
    table_entries = notion_api.query_table(database_id, query)

    assert len(table_entries) == 1
    assert table_entries[0][expected_property] == expected_value


def test_query_with_multiple_pages(notion_api):
    query = {"filter": {"and": [{"property": "año", "formula": {"number": {"equals": 2023}}},
                                {"property": "mes", "formula": {"number": {"equals": 1}}}]},
             "page_size": 10}

    table_entries = notion_api.query_table(NT_EXPENSES_DB_ID, query)

    assert isinstance(table_entries, List) and all(isinstance(i, Dict) for i in table_entries)
    assert len(table_entries) == 37


@pytest.mark.parametrize("page_id, stable_property, updated_property, payload", [
    # Test with tasks database
    (EXISTING_TEST_TASK_PAGE_ID, "Task", "Focus time",
     json.dumps({"properties": {"Focus time": {"number": random.random()}}})),

    # Test with stats database
    (EXISTING_TEST_STAT_PAGE_ID, "work time", "focus time",
     json.dumps({"properties": {"focus time": {"number": random.random()}}})),

    # Test with expenses database
    (EXISTING_TEST_EXPENSE_LOG_PAGE_ID, "producto", "egresos",
     json.dumps({"properties": {"egresos": {"number": random.random()}}})),
])
def test_update_table_entry(notion_api, stable_property, updated_property, page_id, payload):
    original_table_entry = notion_api.get_table_entry(page_id)

    notion_api.update_table_entry(page_id, payload)

    updated_table_entry = notion_api.get_table_entry(page_id)
    assert original_table_entry["properties"][stable_property] == updated_table_entry["properties"][stable_property]
    assert original_table_entry["properties"][updated_property] != updated_table_entry["properties"][updated_property]



import os

NT_AUTH = os.getenv("NT_AUTH", default="")
NT_TASKS_DB_ID = os.getenv("NT_TASKS_DB_ID", default="")
NT_STATS_DB_ID = os.getenv("NT_STATS_DB_ID", default="")
NT_EXPENSES_DB_ID = os.getenv("NT_EXPENSES_DB_ID", default="")



import json
import datetime
from typing import Optional

from tickthon import Task

from ._config import NT_TASKS_DB_ID, NT_EXPENSES_DB_ID, NT_STATS_DB_ID
from ._notion_table_headers import TasksHeaders, ExpensesHeaders, StatsHeaders
from .expense_log_model import ExpenseLog
from .personal_stats_model import PersonalStats


class NotionPayloads:

    @staticmethod
    def get_active_tasks() -> dict:
        return {
            "filter": {
                "and": [
                    {
                        "property": TasksHeaders.DONE.value,
                        "checkbox": {
                            "equals": False
                        }
                    }
                ]
            }
        }

    @staticmethod
    def _base_task_payload(task: Task) -> dict:
        payload = {
            "properties": {
                TasksHeaders.DONE.value: {"checkbox": task.status != 0},
                "title": {"title": [{"text": {"content": task.title}}]},
                TasksHeaders.FOCUS_TIME.value: {"number": task.focus_time},
                TasksHeaders.TAGS.value: {"multi_select": list(map(lambda tag: {"name": tag}, task.tags))},
                TasksHeaders.TICKTICK_ID.value: {"rich_text": [{"text": {"content": task.ticktick_id}}]},
                TasksHeaders.PROJECT_ID.value: {"rich_text": [{"text": {"content": task.project_id}}]},
                TasksHeaders.RECURRENT_ID.value: {"rich_text": [{"text": {"content": task.recurrent_id}}]},
                TasksHeaders.TIMEZONE.value: {"rich_text": [{"text": {"content": task.timezone}}]},
            }
        }

        if task.due_date:
            payload["properties"][TasksHeaders.DUE_DATE.value] = {"date": {"start": task.due_date}}

        return payload

    @classmethod
    def create_task(cls, task: Task) -> str:
        payload = cls._base_task_payload(task)
        payload["parent"] = {"database_id": NT_TASKS_DB_ID}
        return json.dumps(payload)

    @classmethod
    def update_task(cls, task: Task) -> str:
        return json.dumps(cls._base_task_payload(task))

    @staticmethod
    def delete_table_entry() -> str:
        payload = {"archived": True}

        return json.dumps(payload)

    @staticmethod
    def get_task_by_id(task_id: str, due_date: str = "") -> dict:
        """Payload to get a task by its ticktick id.

        Args:
            task_id: The ticktick id of the task. For example: 6f8a2b3c4d5e1f09a7b6c8d9e0f2
            due_date: The due date of the task in format YYYY-MM-DD.
        """
        filters = [{"property": TasksHeaders.TICKTICK_ID.value, "rich_text": {"equals": task_id}}]

        if due_date:
            filters.append({"property": TasksHeaders.DUE_DATE.value, "date": {"equals": due_date}})

        payload = {"sorts": [{"property": TasksHeaders.DUE_DATE.value, "direction": "ascending"}],
                   "filter": {"and": filters}}

        return payload

    @staticmethod
    def create_stat_row(personal_stats: PersonalStats) -> str:
        payload = {
            "parent": {"database_id": NT_STATS_DB_ID},
            "properties": {
                StatsHeaders.DATE.value: {"date": {"start": personal_stats.date}},
                StatsHeaders.WORK_TIME.value: {"number": personal_stats.time_stats.work_time},
                StatsHeaders.LEISURE_TIME.value: {"number": personal_stats.time_stats.leisure_time},
                StatsHeaders.FOCUS_TIME.value: {"number": personal_stats.time_stats.focus_time},
            }
        }

        return json.dumps(payload)

    @staticmethod
    def create_expense_log(expense_log: ExpenseLog) -> str:
        payload = {
            "parent": {"database_id": NT_EXPENSES_DB_ID},
            "properties": {
                ExpensesHeaders.PRODUCTO.value: {"title": [{"text": {"content": expense_log.producto}}]},
                ExpensesHeaders.EGRESOS.value: {"number": expense_log.egresos},
                ExpensesHeaders.FECHA.value: {"date": {"start": expense_log.fecha}}
            }
        }

        return json.dumps(payload)

    @staticmethod
    def get_checked_stats_rows() -> dict:
        payload = {
            "sorts": [{"property": StatsHeaders.DATE.value, "direction": "ascending"}],
            "filter": {"and": [{"property": StatsHeaders.COMPLETED.value, "checkbox": {"equals": True}}]}
        }
        return payload

    @staticmethod
    def get_data_between_dates(initial_date: Optional[datetime.date], today_date: datetime.date) -> dict:
        filters = []
        if initial_date:
            filters.append({"property": "date", "date": {"on_or_after": initial_date.strftime("%Y-%m-%d")}})

        filters.append({"property": "date", "date": {"on_or_before": today_date.strftime("%Y-%m-%d")}})

        return {"sorts": [{"property": "day #", "direction": "ascending"}], "filter": {"and": filters}}

    @staticmethod
    def get_date_rows(date: str) -> dict:
        return {"filter": {"and": [{"property": "date", "date": {"equals": date}}]}}

    @staticmethod
    def update_stat(stat: PersonalStats) -> str:
        payload = {
            "properties": {
                StatsHeaders.DATE.value: {"date": {"start": stat.date}},
                StatsHeaders.WEIGHT.value: {"number": stat.weight},
                StatsHeaders.WORK_TIME.value: {"number": stat.time_stats.work_time},
                StatsHeaders.LEISURE_TIME.value: {"number": stat.time_stats.leisure_time},
                StatsHeaders.FOCUS_TIME.value: {"number": stat.time_stats.focus_time},
            }
        }

        return json.dumps(payload)



from attrs import define


@define
class ExpenseLog:
    """Represents a Ticktick task.

    Attributes:
        fecha: The title of the task, in format YYYY-MM-DD.
        egresos: The amount of money spent.
        producto: The product or service bought.
    """
    fecha: str
    egresos: float
    producto: str
